﻿Imports System
Imports System.Runtime.InteropServices

Friend Module CP2104_Control
    ' Needed to create a handle to access the CP2104
    <DllImport("kernel32.dll")> Private Function _
    CreateFile(ByVal lpFileName As String, _
        ByVal dwDesiredAccess As Integer, ByVal dwShareMode As Integer, _
        ByVal lpSecurityAttributes As Integer, ByVal dwCreationDisposition As Integer, _
        ByVal dwFlagsAndAttributes As Integer, ByVal hTemplateFile As Integer) As Integer
    End Function
    ' Needed to close the handle 
    <DllImport("kernel32.dll")> Private Function _
         CloseHandle(ByVal hObject As Integer) As Integer
    End Function
    ' For more information please check http://msdn.microsoft.com/en-us/library/windows/desktop/aa363858%28v=vs.85%29.aspx

    ' constants needed for "CreateFile" 
    Public Const GENERIC_ALL As Integer = &H10000000
    Public Const GENERIC_READ As Integer = &H80000000
    Public Const GENERIC_WRITE As Integer = &H40000000
    Public Const GENERIC_EXECUTE As Integer = &H20000000
    Public Const OPEN_EXISTING As Integer = 3
    Public Const FILE_SHARE_READ = &H1
    Public Const FILE_SHARE_WRITE = &H2
    Public Const FILE_ATTRIBUTE_NORMAL = &H80
    Public Const FILE_FLAG_OVERLAPPED = &H40000000

    'Variables used within the project
    Dim hUSBDevice As Integer = CP210x_INVALID_HANDLE  'global handle that is set when connected with the CP210x device

    ' read actual relay switch states (more exactly: CP2104 GPIO states) 
    Public Function Read_Status() As Byte
        Dim latch_state_recieved As Byte
        Dim relais_states As Byte = &HFF
        Dim sPort As String

        Try
            If GetSetting("USB Relaycard Tray", "Configuration", "COM") <> "" Then
                sPort = "\\.\" + GetSetting("USB Relaycard Tray", "Configuration", "COM")
                hUSBDevice = CreateFile(sPort, GENERIC_READ Or GENERIC_WRITE, 0, 0, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL Or FILE_FLAG_OVERLAPPED, 0)
                CP210xRT_ReadLatch(hUSBDevice, latch_state_recieved)    ' Read CP2104 GPIO-State to "latch_state_recieved"
                CloseHandle(hUSBDevice)
                relais_states = &HF And (Not (latch_state_recieved))    ' invert recieved state (relays are switched on at low state) and mask out high-nibble
            Else
                MessageBox.Show("No Relay Card Selected", "Error")
            End If

        Catch ex As Exception
            ' MessageBox.Show(Err.Description, "Error")
        End Try
        Return relais_states
    End Function

    ' Switch relays according to checkbox-states
    Public Sub SET_OUTPUTS(ByVal R1 As Byte, ByVal R2 As Byte, ByVal R3 As Byte, ByVal R4 As Byte)
        Dim i As Byte
        Dim sPort As String

        Try
            ' Calculate new  CP2104 output latch value according choosen relays
            If (R1) Then i = i Or CP210x_GPIO_0
            If (R2) Then i = i Or CP210x_GPIO_1
            If (R3) Then i = i Or CP210x_GPIO_2
            If (R4) Then i = i Or CP210x_GPIO_3
            i = i Xor &HFF

            'Write new CP2104 output latch value according choosen relays to choosen board
            If GetSetting("USB Relaycard Tray", "Configuration", "COM") <> "" Then
                sPort = "\\.\" + GetSetting("USB Relaycard Tray", "Configuration", "COM")
                hUSBDevice = CreateFile(sPort, GENERIC_READ Or GENERIC_WRITE, 0, 0, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL Or FILE_FLAG_OVERLAPPED, 0)    'create handle (WIN32 API...)
                CP210xRT_WriteLatch(hUSBDevice, &HF, i)  ' Write new GPIO setting to switch the relays. 0F(hex) => all GPIOs will be addressed
                CloseHandle(hUSBDevice)
                Read_Status()
            Else
                MessageBox.Show("No Relay-Card Selected", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error)
            End If
        Catch ex As Exception
            MessageBox.Show(Err.Description, "Error")
        End Try

    End Sub

    ' List all connected USB Relay Cards (more exactly: all CP2104 devices). Relays are switched on at low state.
    Public Function List_All_Connected_Cp2104() As String()
        Dim sPort As String  'string representing port
        Dim device_type As Byte
        Dim serial_num As String = Space(8)
        Dim device_string As String = Space(50)
        Dim x As Byte = 0
        Dim ReturnStrings(0) As String

        Try
            For i As Byte = 1 To 254
                sPort = "\\.\COM" + i.ToString
                hUSBDevice = CreateFile(sPort, GENERIC_READ Or GENERIC_WRITE, 0, 0, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL Or FILE_FLAG_OVERLAPPED, 0)
                If hUSBDevice <> -1 Then    ' Got a correct handle
                    If CP210x_SUCCESS = CP210xRT_GetPartNumber(hUSBDevice, device_type) Then ' Read Part-Nr using the handle
                        If device_type = CP210x_CP2104_VERSION Then
                            x += 1
                            ReDim Preserve ReturnStrings(x - 1)
                            ReturnStrings(x - 1) = "COM" + i.ToString
                        End If
                    End If
                End If
                CloseHandle(hUSBDevice)
            Next

            If ReturnStrings(0) = Nothing Then
                '   MsgBox("No relay Cards Detected!" + vbNewLine + "- Driver installed?" + vbNewLine + "- Card Connected?" + vbNewLine + "- VREL Jumper Setting OK?", MsgBoxStyle.Critical, "ERROR!")
                ReturnStrings(0) = "ERROR"
            Else

            End If


        Catch ex As Exception
            MessageBox.Show(ex.Message, "Error")
        End Try

        Return ReturnStrings

    End Function




End Module
